
/*]  Copyright (c) 2009-2010, Charles McGarvey  [**************************
**]  All rights reserved.
*
* vi:ts=4 sw=4 tw=75
*
* Distributable under the terms and conditions of the 2-clause BSD license;
* see the file COPYING for a complete text of the license.
*
**************************************************************************/

#ifndef _GAMELAYER_HH_
#define _GAMELAYER_HH_

/**
 * @file GameLayer.hh
 * This is the big enchilada.
 */

#include <iostream>
#include <string>
#include <vector>

#include <boost/shared_ptr.hpp>

#include <Moof/Layer.hh>
#include <Moof/Math.hh>
#include <Moof/Sound.hh>

#include <Moof/Line.hh>		// TODO
#include <Moof/Plane.hh>
#include <Moof/Ray.hh>
#include <Moof/Sphere.hh>
#include <Moof/Timer.hh>

#include "GameState.hh"
#include "Hud.hh"


class GameLayer;
typedef boost::shared_ptr<GameLayer> GameLayerP;

class GameLayer : public Mf::Layer
{
public:

	GameLayer();

	static GameLayerP alloc()
	{
		return GameLayerP(new GameLayer);
	}

	void addedToCore();

	void update(Mf::Scalar t, Mf::Scalar dt);
	void draw(Mf::Scalar alpha) const;
	bool handleEvent(const Mf::Event& event);

private:

	void loadSceneLoader();
	void advanceScene();

	void thinkTimer();

	void setProjection();
	void setProjection(Mf::Scalar width, Mf::Scalar height);

	GameState		mState;
	Mf::Timer		mThinkTimer;

	HudP			mHud;

	Mf::SoundStream	mMusic;
	Mf::Sound 		mPunchSound;

	Mf::Ray2		mRay;
	Mf::Line2		mLine;
	Mf::Circle		mCircle;

	Mf::Timer		mRayTimer;
	void rayTimer();
};


#endif // _GAMELAYER_HH_

